;*****************************************************************
;**	     Gameboy full screen distortion demo 		**
;** 		  (c) Anders Granlund 1997			**
;*****************************************************************
;** The gameboy seems to be able to do the most cool things.	**
;** This is just a simple example of what can be done with the	**
;** interrupts..						**
;** It makes use of a sine-table, so there's not any realtime	**
;** calculations.						**
;**								**
;** Credits to Dr. Pan of anthrox for his Sine-table generator	**
;** "Rolleroaster" !						**
;*****************************************************************

memvar				.EQU $CC00
distpos				.EQU memvar
ypos				.EQU memvar+1
bary				.EQU memvar+2

.org	0

.org $40
  jp Vblank_interrupt
  reti
.org $48
  jp lcd_interrupt
  reti
.org $50
  reti
.org $58
  reti
.org $60
  reti


.org $100
  nop
  jp begin

;Nintendo Scrolling Title Graphic
;***********************************************
.byte	0ceh,0edh,66h,66h,0cch,13,0,11,3,73h
.byte	0,83h,0,12,0,13,0,8,17,31,88h,89h,0
.byte	14,0dch,0cch,6eh,0e6h,0ddh,0ddh,0d9h
.byte	99h,0bbh,0bbh,67h,63h,6eh,0eh,0ech,0cch
.byte	0ddh,0dch,99h,09fh,0bbh,0b9h,33h,3eh
;***********************************************

.byte   "Distortion      "              ;Title of the game
.byte	0,0,0				;Not used
.byte	0 				;Cartridge type: Rom only
					; 0 - Rom only      3 - ROM+MBC1+RAM+Battery
					; 1 - ROM+MBC1      5 - ROM+MBC2
					; 2 - ROM+MBC1+RAM  6 - ROM+MBC2+Battery
.byte	0				;Rom Size:
					; 0 - 256kBit =  32kB =  2 banks
					; 1 - 512kBit =  64kB =  4 banks
					; 2 -   1MBit = 128kB =  8 banks
					; 3 -   2MBit = 256kB = 16 banks
					; 4 -   4MBit = 512kB = 32 banks
.byte   0				;Ram Size:
					; 0 - None
					; 1 -  16kBit =  2kB = 1 bank
					; 2 -  64kBit =  8kB = 1 bank
					; 3 - 256kBit = 32kB = 4 banks
.byte	"Vi","ka"			;Manufacturer code:
.byte	1				;Version Number
.byte	0ah				;Complement check
.word	0				;Checksum

;***********
;** SETUP **
;***********
begin:
  di
  ld a,20
  ld (bary),a
  ld a,0
  ld (distpos),a
  ld (ypos),a
  ldh ($42),a			;Scroll X
  ldh ($43),a			;Scroll Y


  ld a,%10010011
  ldh ($40),a			;Turn screen on


  ld a,%11100100		;"normal" colours
  ldh ($47),a			;BG palette
  ldh ($48),a			;OBJ0 palette
  ldh ($49),a			;OBJ1 palette
  ld a,0			;First ldc-int at Y=0
  ldh ($45),a
  ld a,%01000100		;set lcdc int
  ldh ($41),a
  ld a,%00000011
  ldh ($FF),a			;We want LCDC int + Vblank
 
  call load_map			;Load the background map
  call load_tiles		;Load tiles

  ei				;Enable those int's!


;***************
;** MAIN LOOP **
;***************


main:
  jp main



;********************
;** LCDC-INTERRUPT **
;********************

lcd_interrupt:
  push af				;Save A + flags
;  push bc
;  push de
;  push hl
  ld hl,distdata			;Set HL at first X-value
  ld a,(ypos)				;What line are we at?
  ld b,0
  ld d,b
  ld e,a
  add hl,de				;Set HL to point at right value

  ld a,(distpos)
  ld b,0
  ld d,b
  ld e,a
  add hl,de

  ld a,(hl)
  srl a
  ldh ($43),a				;Scroll line
  ld a,(ypos)
  inc a
  cp 144
  call z,resetA
  ld (ypos),a
  ldh ($45),a				;Set next ldc-int to occur at next line

  ld a,(ypos)
  ld b,a
  ld a,(bary)
  cp b
  jr z,bar1
  inc a
  cp b
  jr z,bar2
  inc a
  cp b
  jr z,bar3
  inc a
  cp b
  jr z,bar4
  inc a
  cp b
  jr z,bar5
  inc a
  cp b
  jr z,bar6
  inc a
  cp b
  jr z,bar7
  inc a
  cp b
  jr z,endbar
end_lcdcint:
;  pop hl
;  pop de
;  pop bc
  pop af				;restore a + flags
  reti
bar1:
  ld a,%11111111
  ldh ($47),a
  jr end_lcdcint
bar2:
  ld a,%10101010
  ldh ($47),a
  jr end_lcdcint
bar3:
  ld a,%01010101
  ldh ($47),a
  jr end_lcdcint
bar4:
  ld a,%00000000
  ldh ($47),a
  jr end_lcdcint
bar5:
  ld a,%01010101
  ldh ($47),a
  jr end_lcdcint
bar6:
  ld a,%10101010
  ldh ($47),a
  jr end_lcdcint
bar7:
  ld a,%11111111
  ldh ($47),a
  jr end_lcdcint
endbar:
  ld a,%11100100
  ldh ($47),a
  jr end_lcdcint
resetA:
  ld a,0
  ret



;**********************
;** VBLANK-INTERRUPT **
;**********************

Vblank_interrupt:
  push af

  ld hl,distdata2		;Move the rasterbar
  ld a,(distpos)
  ld d,0
  ld e,a
  add hl,de
  ld a,(hl)
  ld (bary),a   

  ld a,(distpos)
  inc a
  cp 145
  jr nz,end_vblankint
  ld a,0
end_vblankint:
  ld (distpos),a
  pop af
  reti




;******************
;** SUB-ROUTINES **
;******************

load_map:
  ld de,$9800			;Mapmem location
  ld hl,map			;Mapdata
  ld c,32			;Height=32
ldmap1:
  ld b,32			;Width=32
ldmap2:
  ld a,(hli)
  call wrAtode
  inc de
  dec b
  jr nz,ldmap2
  dec c
  jr nz,ldmap1
  ret


load_tiles:
  ld de,$8000			;BG-tiles position
  ld hl,tiles			;Tile-data to load
  ld c,255			;Nr. of tiles
loadtiles:
  ld b,16			;16 bytes per tile 
loadonetile:
  ld a,(hli)
  call wrAtode
  inc de
  dec b
  jr nz,loadonetile
  dec c
  jr nz,loadtiles
  ret


wrAtode:		;Writes A to HL at the right time
  push af		;Save reg A and flags
wral2:
  ldh a,($41)
  and 2
  jr nz,wral2
  pop af		;Restore reg A and flags
  ld (de),a		;write a to hl
  ret


;**********
;** DATA **
;**********

tiles:
#include "dist.z80"			;The tiles
distdata:
#include "distdata.z80"			;The sin-table
distdata2:
#include "distdat2.z80"			;Table for rasterbar
map:
#include "distmap.z80"			;Screen mapdata

.end
